@@ -837,12 +837,13 @@ sunone_connection_finalize (GObject *object)
 }
 
 SunOneConnection *
-sunone_connection_new (const char *server_uri,
+sunone_connection_new (const char *server_uri, const char *proxy_uri,
 		       const char *user,
 		       const char *password)
 {
 	SunOneConnection *connection;
 	SunOneConnectionPrivate *priv;
+	SoupUri *suri;
 
 	connection = g_object_new (SUNONE_CONNECTION_TYPE, NULL);
 	priv = connection->priv;
@@ -850,6 +851,11 @@ sunone_connection_new (const char *server_uri,
 	priv->user = g_strdup (user);
 	priv->password = g_strdup (password);
 	priv->is_open = FALSE;
+	if (proxy_uri && priv->soup_session) {
+		suri = soup_uri_new (proxy_uri);
+		g_object_set (G_OBJECT (priv->soup_session), SOUP_SESSION_PROXY_URI, suri, NULL);
+		soup_uri_free (suri);
+	}
 	
 	return connection;
 }
@@ -915,6 +921,20 @@ clone_prefs (SunOneConnectionPreferences *prefs)
 	return copy;
 }
 
+static gboolean
+already_in_subscriptions (GList *subscriptions, const char *calid)
+{
+	GList *l;
+
+	for (l = subscriptions; l != NULL; l = l->next) {
+		SunOneCalendarSubscription *sub = l->data;
+		if (!strcmp (sub->calid, calid)) 
+			return TRUE;
+	}
+
+	return FALSE;
+}
+
 static void
 parse_subscriptions (SunOneConnection *connection, const gchar *val)
 {
@@ -932,7 +952,7 @@ parse_subscriptions (SunOneConnection *connection, const gchar *val)
 		parts = g_strsplit (items[n], "$", 0);
 
 		if (parts) {
-			if (parts[0] != NULL) {
+			if (parts[0] != NULL && !already_in_subscriptions (priv->prefs->subscriptions, parts[0])) {
 				sub = g_new0 (SunOneCalendarSubscription, 1);
 				sub->calid = g_strdup (parts[0]);
 				if (parts[1] != NULL)
@@ -988,9 +1008,41 @@ sunone_connection_set_user (SunOneConnection *connection, const char *user)
 	SunOneConnectionPrivate *priv = connection->priv;
 
 	g_return_if_fail (IS_SUNONE_CONNECTION (connection));
+
+	g_free (priv->user);
 	priv->user = g_strdup (user);
 }
 
+static void
+check_default_calendar (SunOneConnection *connection)
+{
+	SunOneConnectionPrivate *priv = connection->priv;
+	GList *l;
+	SunOneCalendarSubscription *sub;
+	gboolean found = FALSE;
+
+	/* Not great but it is possible to have an empty icsCalendar value */
+	if (!priv->prefs->default_calendar)
+		priv->prefs->default_calendar = g_strdup (priv->user);
+
+	/* Make sure default_calendar is in subscriptions */
+	for (l = priv->prefs->subscriptions; l!= NULL; l = l->next) {
+		sub = l->data;
+		if (!strcmp (priv->prefs->default_calendar, sub->calid)) {
+			found = TRUE;
+			break;
+		}
+	}
+
+	if (!found) {
+		sub = g_new0 (SunOneCalendarSubscription, 1);
+		sub->calid = g_strdup (priv->prefs->default_calendar);
+		sub->display_name = g_strdup (priv->prefs->default_calendar);
+		priv->prefs->subscriptions = g_list_append (
+					priv->prefs->subscriptions, sub);
+	}
+}
+
 SunOneConnectionPreferences *
 sunone_connection_get_preferences (SunOneConnection *connection)
 {
@@ -1086,10 +1138,7 @@ sunone_connection_get_preferences (SunOneConnection *connection)
 
 	icalcomponent_free (icalcomp);
 
-	/* Not great but it is possible to have an empty icsCalendar value */
-	if (!priv->prefs->default_calendar)
-		priv->prefs->default_calendar = g_strdup (priv->user);
-
+	check_default_calendar (connection);
 	clone = clone_prefs (priv->prefs);
 	g_static_mutex_unlock (&mutex);
 
