@@ -33,6 +33,7 @@ struct _SunOneConfigListenerPrivate {
 enum {
 	SUNONE_ACCOUNT_CREATED,
 	SUNONE_ACCOUNT_REMOVED,
+	SUNONE_ACCOUNT_CHANGED,
 	LAST_SIGNAL
 };
 
@@ -84,6 +85,15 @@ class_init (GObjectClass *object_class)
 				sunone_marshal_NONE__POINTER,
 				G_TYPE_NONE, 1,
 				G_TYPE_POINTER);
+	signals[SUNONE_ACCOUNT_CHANGED] =
+		g_signal_new ("sunone_account_changed",
+				G_OBJECT_CLASS_TYPE (object_class),
+				G_SIGNAL_RUN_LAST,
+				G_STRUCT_OFFSET (SunOneConfigListenerClass, sunone_account_changed),
+				NULL, NULL,
+				sunone_marshal_NONE__POINTER,
+				G_TYPE_NONE, 1,
+				G_TYPE_POINTER);
 }
 
 static void 
@@ -184,6 +194,9 @@ add_esource (SunOneAccount *account, const char *folder_name,
 			ret = e_source_group_add_source (source_group, source, -1);
 			
 			g_object_unref (source);
+		} else {
+			/*If exists, update relative_uri only*/
+			e_source_set_relative_uri (source, relative_uri);
 		}
 	}
 
@@ -300,6 +313,73 @@ add_source (SunOneAccount *account, const char *calid, const char *display_name,
 	}
 }
 
+static void
+remove_unused_sources(SunOneAccount *account, SunOneConnectionPreferences *preference)
+{
+	SunOneCalendarSubscription *sub;
+	ESourceGroup *group;
+	const char *source_name;
+	const char *pref_name;
+	GSList *sources;
+
+	g_return_if_fail ((account != NULL) && (preference != NULL));
+
+	group = e_source_list_peek_group_by_name (source_list_cal,
+						sunone_account_get_name (account));
+	if (group) {
+		for (sources = e_source_group_peek_sources (group); sources != NULL; sources = sources->next) {
+			ESource *source = E_SOURCE (sources->data);
+			GList *prefs;
+			char *physical_uri;
+
+			if (!source)
+				continue;
+
+			source_name = e_source_peek_name (source);
+			for (prefs = preference->subscriptions; prefs != NULL; prefs = prefs->next) {
+				SunOneCalendarSubscription *sub = (SunOneCalendarSubscription *) prefs->data;
+				if (!sub)
+					continue;
+
+				pref_name = sub->display_name && *(sub->display_name) ? sub->display_name : sub->calid;
+				if (!strcmp (source_name, pref_name))
+					break;
+			}
+			if (!prefs)
+				/*could not find the subscription, remove the unused esource*/
+				e_source_group_remove_source_by_uid (group, e_source_peek_uid (source));
+		}
+	}
+
+	group = e_source_list_peek_group_by_name (source_list_task,
+						sunone_account_get_name (account));
+	if (group) {
+		for (sources = e_source_group_peek_sources (group); sources != NULL; sources = sources->next) {
+			ESource *source = E_SOURCE (sources->data);
+			GList *prefs;
+			char *physical_uri;
+
+			if (!source)
+				continue;
+
+			source_name = e_source_peek_name (source);
+			for (prefs = preference->subscriptions; prefs != NULL; prefs = prefs->next) {
+				SunOneCalendarSubscription *sub = (SunOneCalendarSubscription *) prefs->data;
+				if (!sub)
+					continue;
+
+				pref_name = sub->display_name && *(sub->display_name) ? sub->display_name : sub->calid;
+				if (!strcmp (source_name, pref_name))
+					break;
+			}
+			if (!prefs)
+				/*could not find the subscription, remove the unused esource*/
+				e_source_group_remove_source_by_uid (group, e_source_peek_uid (source));
+		}
+	}
+
+}
+
 void
 add_sources (SunOneAccount *account, SunOneConnectionPreferences *prefs)
 {
@@ -311,6 +391,7 @@ add_sources (SunOneAccount *account, SunOneConnectionPreferences *prefs)
 
 	g_return_if_fail (account != NULL);
 
+	remove_unused_sources (account, prefs);
 	for (l = prefs->subscriptions; l != NULL; l = l->next) {
 		sub = (SunOneCalendarSubscription *) l->data;
 		if (!sub)
@@ -325,25 +406,6 @@ add_sources (SunOneAccount *account, SunOneConnectionPreferences *prefs)
 		physical_uri = sunone_util_get_physical_uri_from_account (account, calid, "tasks");
 		add_esource (account, display_name, physical_uri, &source_list_task);
 		g_free (physical_uri);
-#if 0
-		if (!strcasecmp (sub->calid, prefs->default_calendar)) {
-			physical_uri = sunone_util_get_physical_uri_from_account (account, calid, "calendar");
-			add_esource (account, _("Personal Calendar"), physical_uri, &source_list_cal);
-			g_free (physical_uri);
-
-			physical_uri = sunone_util_get_physical_uri_from_account (account, calid, "tasks");
-			add_esource (account, _("Personal Task"), physical_uri, &source_list_task);
-			g_free (physical_uri);
-		} else {
-			physical_uri = sunone_util_get_physical_uri_from_account (account, calid, "calendar");
-			add_esource (account, display_name, physical_uri, &source_list_cal);
-			g_free (physical_uri);
-
-			physical_uri = sunone_util_get_physical_uri_from_account (account, calid, "tasks");
-			add_esource (account, display_name, physical_uri, &source_list_task);
-			g_free (physical_uri);
-		}
-#endif
 	}
 }
 
@@ -379,14 +441,12 @@ rename_source (SunOneAccount *account, const char *calid, const char *display_na
 static void
 add_account (EAccountList *account_list, EAccount *account)
 {
-	SunOneConfigListener *config_listener;
+	SunOneConfigListener *config_listener = SUNONE_CONFIG_LISTENER (account_list);
 	SunOneAccount *sunone_account;
 
 	if (!is_active_sunone_account (account))
 		return;
 
-	config_listener = SUNONE_CONFIG_LISTENER (account_list);
-
 	/* New account! Yippee! */
 	sunone_account = sunone_account_new (account_list, account);
 	if (!sunone_account) {
@@ -404,34 +464,39 @@ add_account (EAccountList *account_list, EAccount *account)
 static void
 change_account (EAccountList *account_list, EAccount *account)
 {
-	SunOneAccount *sunone_account;
+	SunOneAccount *old_account;
 
-	sunone_account =  sunone_component_get_account_from_name (global_sunone_component, account->name);
+	old_account =  sunone_component_get_account_from_uid (global_sunone_component, account->uid);
 
-	if (!sunone_account) {
-		if (!is_active_sunone_account (account))
-			return;
-
-		/* The user has converted an existing non-SunOne
-		 * account to a SunOne account, so treat it like an
-		 * add.
+	if (old_account && !is_active_sunone_account (account)) {
+		/* If account is removed or settings is changed,
+		 * remove the old sunone account.
+		 */
+		g_object_unref (old_account);
+		remove_account (account_list, account);
+	} else if (!old_account && is_active_sunone_account (account)) {
+		/* Converted from a non-sunone account or 
+		 * settings was changed, create a new one.
 		 */
 		add_account (account_list, account);
-	} else  {
-		g_object_unref (sunone_account);
-		if (!is_active_sunone_account (account)) {
-			/* The user has disabled the SunOne account or
-			 * converted it to non-SunOne, so treat it like a
-			 * remove.
-			 */
-			remove_account (account_list, account);
-			return;
+	} else if (is_active_sunone_account (account)){
+		SunOneAccount *sunone_account;
+
+		g_object_unref (old_account);
+		sunone_account = sunone_account_new (account_list, account);
+		if (sunone_account) {
+			g_signal_emit (SUNONE_CONFIG_LISTENER (account_list), signals[SUNONE_ACCOUNT_CHANGED], 0,
+					   sunone_account);
+
+			g_object_unref (sunone_account);
+
+			e_notice (NULL, GTK_MESSAGE_WARNING,
+				  _("Some changes to Sun JESCS account configuration will "
+					"take\nplace after you quit and restart Evolution."));
 		}
+
 	}
 
-	e_notice (NULL, GTK_MESSAGE_WARNING,
-		  _("Some changes to Sun JESCS account configuration will "
-		    "take\nplace after you quit and restart Evolution."));
 	return;
 
 }
@@ -442,7 +507,7 @@ remove_account (EAccountList *account_list, EAccount *account)
 	SunOneConfigListener *config_listener =	SUNONE_CONFIG_LISTENER (account_list);
 	SunOneAccount *sunone_account;
 
-	sunone_account =  sunone_component_get_account_from_name (global_sunone_component, account->name);
+	sunone_account =  sunone_component_get_account_from_uid (global_sunone_component, account->uid);
 
 	/* It might not be a sunone account */
 	if (!sunone_account)
